/*****************  Standard C Source Header ****************
*
* PLEASE SET YOUR EDITOR FOR 4 TABS
*
*
* **********************************************************************************
*
* GENERAL INFORMATION
*
*   Module Description:     Serial LCD driver library for PICVUE-064 with AVR controller
                            Serial LCD Library for PICVUE PC-064-PYC 2x8 Character LCD module
                            Uses HD44780 compatible controller.

                            Example program using the PICVUE LCD
							As of 050315


        This library can be used with any LCD character module using a HD44780 controller
        and two each 74HC164 shift registers.
*
*
*   File Name........... SER_LCD.C  - LCD support functions
*
*   VERSION NO.......... 1.01
*   PRIMARY AUTHOR ..... GJO
*   OTHER AUTHORS ...... GJO
*   START DATE ......... Feb 24, 2005
*   RELEASE DATE........
*   JOB NUMBER.......... n/a
*   PROJECT NAME........ n/a
*   PRODUCT NAME........ n/a
*   PRODUCT NUMBER...... n/a
*   COMMENTS............
*
* FORMAL DOCUMENTATION
*
*   PROGRAMMER'S MANUAL ........... None
*   NAME OF FILE CONTAINED IN...... n/a
*   COMMENTS....................... In source file only
*   USER'S MANUAL ................. None
*
*   LIST ANY OTHER REFERENCE DOCUMENTS
*
*   Custom graphics code based on PC interface examples by Randy Rasa:
*   * Copyright 1996 by Randy Rasa
*   *                   rrasa@sky.net
*
*   Dependencies:   ser_LCD_V101.h, fonts.h
*
*   Revision list:
*
*   rev	    date		who		description
*   ----	-------		---		------------------------------------------------
*   1.00	05.02.26	GJO		Wrote library and tested functionality
*   1.01    05.03.09    GJO     Added custom graphics loader
*
    LCD User Functions included:
 ===============================================================================

    line(char which_line);                  // Set current line
    clear_display(void);                    // Clear LCD screen

    set_LCD_cur(LCDrow, LCDcol);            // Set new row and cursor position

    wr_disp_ser(data)                       // Low level LCD write function

    disp_char(c);                           // Display single ASCII character
    disp_str(*sa);                          // Display string by RAM pointer
    disp_cstr(*sa);                         // Display string by ROM pointer

    LCD_CursorLeft ();                      // Move cursor left
    LCD_CursorRight ();                     // Move Cursor Right
    LCD_CursorOn ();                        // Turn Cursor On
    LCD_CursorOff ();                       // Turn Cursor Off
    LCD_DisplayOff ();                      // Turn Display Off
    LCD_DisplayOn ();                       // Turn Display On

*/

//********************************************************************
// System library & functions
//********************************************************************
#include <MEGA32.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <delay.h>

#include "fonts.h"              // LCD custom graphics font bit maps
#include "SER_LCD_V101.h"       // All Serial PICVUE PC-064-PYH LCD support
#include "SER_LCD_V101.c"       // All Serial PICVUE PC-064-PYH LCD support


/*-----------------------------------------------------------------------
* Serial 2x8 LCD module (Uses HC164 Shift register feeding a HD44780)
* DT003 wiring interface used for prototype testing:
*-----------------------------------------------------------------------
*   Pin     Function            SIM100 Bus          LCD pin layout
*-----------------------------------------------------------------------
*   1       SPI CLK             D7  22       |  Top     o  | CLK
*   2       SPI DATA            D5  20       |  of      o  | DATA
*   3       not used                         |  LCD        | no pin
*   4       GND                     9        |          o  | GND
*   5       E CLK               D4  19       |          o  | E
*   6       +5V                     7        |          o  | +5V
*-----------------------------------------------------------------------
PICVUE-064-HY 2x8 LCD module wiring:
 ------------------------------
 |  __________                |
 | |  _____   |               |              MTA PLUG 5 pin
 | | /     \  |   1 o         | ------------ 1 - Clock
 | | |     |  |   2 o         | ------------ 2 - Data
 | | |     |  |               |
 | | |     |  |   3 o         | ------------ 3 - Ground
 | | |     |  |   4 o         | ------------ 4 - E-strobe
 | | \_____/  |   5 o         | ------------ 5 - +5V @ 90mA
 | |__________|               |
 | (Uses HD44780 controller)  |
 ------------------------------         
    LCD data interface information:

    The data sent to the display is in the format:
    (This is handled by disp_byte_ser(char)automatically)

    See init_SPI() for further information

    SCK _____ (8 clks)   ______            ________________
             \/\/\/\/\/\/      \/\/\/\/\/\/
              __________        __________
    MOSI_____/ RS SEL   \______/ LCD data \________________ SPI data Xfer
             \__________/      \__________/
            MSB (0x80)         MSB        LSB
             |-- 10us --|      |---10us---|     ___
    PC.7  _____________________________________/   \_______ E-strobe to update HD44780
                                         5us
    SPI configuration:  Positive clock polarity
                        LSB sent first
                        Clock Polarity normal
                        Clock speed 1us
*/

//***********************************************************************
//* Initialize AVR chip
//***********************************************************************
void init_AVR(void)
{

    // Input/Output Ports initialization
	// Port A
	PORTA=0x00;
	DDRA=0x00;

	// Port B
	// SS = PB4
	// MOSI = PB5
	// MISO = PB6
	// SCK = PB7
	DDRB=0b10110000;
	PORTB=0b00000000;

	// Port C
	// C.0 - C.7 = OUTPUT
	PORTC=0x03;
	DDRC=0b11111111;

	// Port D
	// D.0=RS232 INPUT 	0
	// D.1=RS232 OUTP	1
    // D.4=SHT POWER    1
    // D.5=SDA (in)		0
	// D.6=SCL (out)	1
	// D.7=SHT_LED      1 (Also E_CLK on SPI LCD)
	PORTD = 0b11000010;
	DDRD  = 0b11111110;


}

//*********************************************************************************************************
void main(void)
{

    char i;
    init_AVR();										// Setup AVR hardware (existing one - not everything applies)
    init_SPI();                                     // Initialize SPI port for PICVUE

    init_display_ser();                             // Initialize all relevant SPI LCD registers
    write_custom_chars_LCD();                       // Load custom defined character bit maps into LCD CGRAM
    clear_display();                                // Clear LCD RAM

	set_LCD_cur(0,1);                               // Set LCD cursor position
	disp_cstr("HELLO");                             // Display Text
    set_LCD_cur(1,1);                               // Set new LCD cursor position
	disp_cstr("WORLD");                             // Display Text

    delay_ms(3000);
    clear_display();                                // Clear LCD RAM
    set_LCD_cur(0,0);                               // Move cursor to line 2
    glrsel=0xFF;                                    // Enter LCD data mode

    for (i=0;i<8;i++)
        wr_disp_ser(i);                             // Display all 8 CG characters

    set_LCD_cur(1,0);                               // Move cursor to line 2
    disp_cstr("Done...");                           // Display firmware version

	for (;;); // Do nothing


}
